// setup global configuration
if (!ul) var ul = {};
if (!ul.tran) ul.tran = {};
if (!ul.tran.lang_def) ul.tran.lang_def = 'eng';
if (!ul.tran.select_id) ul.tran.select_id = 'id_select_lang';
if (!ul.params) ul.params = {};
if (!ul.params.lang) ul.params.lang = ul.tran.lang_def;

// embedded translations splitter regexp (example : word_default_lang[:lang1:]word_lang1[:lang2:]word_lang2)
ul.tran.splitter = '\\[\\:(\\w*)\\:\\]';
if (!ul.tran.attr_lang) ul.tran.attr_lang = 'lang';
if (!ul.tran.attr_tran) ul.tran.attr_tran = 'tran';

// filters/translates string according to selected language lang
// if lang not set, ul.params.lang taken
// if lang is Array, array of present languages appended to it
// if lang is Object, associative array of language : translation returned
function tran_str(str_in, lang, lang_def) {
  if (str_in) {
    if (!lang_def) lang_def = ul.tran.lang_def;
    if (!lang) lang = ul.params.lang;
    var tran_result = {};
    var lang_prev = lang_def;
    var index_prev = 0;
    var str_re = RegExp(ul.tran.splitter, 'gm');
    var s = '';
    while ((str_match = str_re.exec(str_in)) != null) {
      tran_result[lang_prev] = str_in.slice(index_prev, str_match.index);
      index_prev = str_re.lastIndex;
      lang_prev = str_match[1];
    }
    tran_result[lang_prev] = str_in.slice(index_prev);
    if ((typeof lang) == 'string') {
      if (tran_result[lang]) return tran_result[lang];
      else if (tran_result[lang_def]) return tran_result[lang_def];
      else return str_in;
    } else if (lang instanceof Array) {
        var langs = [];
        for (var lng in tran_result) if ((lng != '') && (tran_result[lng] != '')) langs.push(lng);
        for (var lng in lang) if (!(lang[lng] in tran_result)) langs.push(lang[lng]);
        return langs;
      } else if (lang instanceof Object) {
          return tran_result;
        } else return tran_result;
  }
}

// filters/translates childNodes of block_in using language lang and enabled/disabled styles
function tran_childnodes(block_in, lang, enabled_style, disabled_style) {
  function tran_child(node_in) {
    if ((node_in) && (node_in.getAttribute)) {
      if (node_in.getAttribute(ul.tran.attr_lang)) {
        if (lang.toString().indexOf(node_in.getAttribute(ul.tran.attr_lang)) == -1) node_in.style.display = disabled_style;
          else node_in.style.display = enabled_style;
      } else if (node_in.getAttribute(ul.tran.attr_tran)) {
          node_in.innerHTML = tran_str(node_in.getAttribute(ul.tran.attr_tran), lang);
        }
    }
  }
  if (!lang) lang = ul.params.lang;
  if (!enabled_style) enabled_style = 'inline';
  if (!disabled_style) disabled_style = 'none';
  dom_childnodes_walk(block_in, tran_child); // translate all child nodes
}

// returns array of used languages in given element and it's subelements
function tran_langs(block_in) {
  var langs = [];
  function tran_child_langs(node_in) {
    if ((node_in) && (node_in.getAttribute)) {
      if (node_in.getAttribute(ul.tran.attr_lang)) {
        if (array_indexOf(langs, node_in.getAttribute(ul.tran.attr_lang)) == -1) langs.push(node_in.getAttribute(ul.tran.attr_lang));
      } else if (node_in.getAttribute(ul.tran.attr_tran)) {
          langs = tran_str(node_in.getAttribute(ul.tran.attr_tran), langs);
        }
    }
  }
  dom_childnodes_walk(block_in, tran_child_langs); // search children nodes for language attribute
  return(langs);
}

// startup document translation and language select creation
function tran_onload() {

  function lang_sel_changed() {
    ul.params.lang = this.value;
    tran_childnodes(document.body);
  }

  if ((langs_sel = document.getElementById(ul.tran.select_id)) != null) {
    var langs = tran_langs(document.body);
    for (var lng in langs)
      langs_sel.options.add(new Option(langs[lng], langs[lng], (langs[lng] == ul.params.lang), (langs[lng] == ul.params.lang)));
    langs_sel.onchange = lang_sel_changed;
  }
  tran_childnodes(document.body);
}

ul.onload.push(tran_onload);
